@@ -13,7 +13,7 @@ PREFIX = colorstr('Ultralytics: ')
 def train_model(api_key='') -> None:
     """Starts training from next in queue"""
     clone_yolo()
-    connect_to_hub(api_key)
+    connect_to_hub(api_key=api_key)
     trainer = Trainer(None, AUTH)  # No model so next in train queue is fetched
     if trainer.model is not None:
         trainer.start()
@@ -26,6 +26,6 @@ def connect_to_hub(api_key='', verbose=False) -> bool:
     if CONNECTED and verbose:
         print(f'{PREFIX}Already logged in.')
     elif not CONNECTED:
-        CONNECTED = AUTH.attempt_api_key(api_key)
+        CONNECTED = AUTH.attempt_api_key(api_key=api_key)
 
     return CONNECTED
